@@ -5,21 +5,25 @@ module Local_package = Dune_pkg.Local_package
 module Show_lock = struct
   let print_lock lock_dir_arg () =
     let open Fiber.O in
-    let+ lock_dir_paths =
+    let* lock_dir_paths =
       Memo.run (Workspace.workspace ())
       >>| Pkg_common.Lock_dirs_arg.lock_dirs_of_workspace lock_dir_arg
     in
-    Console.print
-    @@ List.map lock_dir_paths ~f:(fun lock_dir_path ->
+    Fiber.parallel_map lock_dir_paths ~f:(fun lock_dir_path ->
+      let+ platform = Pkg_common.solver_env_from_system_and_context ~lock_dir_path in
       let lock_dir = Lock_dir.read_disk_exn lock_dir_path in
+      let packages =
+        Lock_dir.Packages.pkgs_on_platform_by_name lock_dir.packages ~platform
+        |> Package_name.Map.values
+      in
       Pp.concat
         ~sep:Pp.space
         [ Pp.hovbox
           @@ Pp.textf "Contents of %s:" (Path.Source.to_string_maybe_quoted lock_dir_path)
-        ; Pkg_common.pp_packages
-            (Package_name.Map.to_list_map ~f:(fun _ pkg -> pkg) lock_dir.packages)
+        ; Pkg_common.pp_packages packages
         ]
       |> Pp.vbox)
+    >>| Console.print
   ;;
 
   let term =
@@ -135,33 +139,33 @@ module List_locked_dependencies = struct
 
   let list_locked_dependencies ~transitive ~lock_dirs () =
     let open Fiber.O in
-    let+ lock_dirs_by_path, local_packages =
+    let* lock_dirs_by_path, local_packages =
       let open Memo.O in
       Memo.both
         (Workspace.workspace () >>| enumerate_lock_dirs_by_path ~lock_dirs)
         Pkg_common.find_local_packages
       |> Memo.run
     in
-    let pp =
-      Pp.concat
-        ~sep:Pp.cut
-        (List.map lock_dirs_by_path ~f:(fun (lock_dir_path, lock_dir) ->
-           let package_universe =
-             Package_universe.create local_packages lock_dir |> User_error.ok_exn
-           in
-           Pp.vbox
-             (Pp.concat
-                ~sep:Pp.cut
-                [ Pp.hbox
-                    (Pp.textf
-                       "Dependencies of local packages locked in %s"
-                       (Path.Source.to_string_maybe_quoted lock_dir_path))
-                ; Pp.enumerate
-                    (Package_name.Map.keys local_packages)
-                    ~f:(package_deps_in_lock_dir_pp package_universe ~transitive)
-                  |> Pp.box
-                ])))
-      |> Pp.vbox
+    let+ pp =
+      Fiber.parallel_map lock_dirs_by_path ~f:(fun (lock_dir_path, lock_dir) ->
+        let+ platform = Pkg_common.solver_env_from_system_and_context ~lock_dir_path in
+        let package_universe =
+          Package_universe.create ~platform local_packages lock_dir |> User_error.ok_exn
+        in
+        Pp.vbox
+          (Pp.concat
+             ~sep:Pp.cut
+             [ Pp.hbox
+                 (Pp.textf
+                    "Dependencies of local packages locked in %s"
+                    (Path.Source.to_string_maybe_quoted lock_dir_path))
+             ; Pp.enumerate
+                 (Package_name.Map.keys local_packages)
+                 ~f:(package_deps_in_lock_dir_pp package_universe ~transitive)
+               |> Pp.box
+             ]))
+      >>| Pp.concat ~sep:Pp.cut
+      >>| Pp.vbox
     in
     Console.print [ pp ]
   ;;
