@@ -159,9 +159,9 @@ module Print_config = struct
     | Kwd -> [ `Bold; `Fg_blue ]
     | Id -> [ `Bold; `Fg_yellow ]
     | Prompt -> [ `Bold; `Fg_green ]
-    | Hint -> [ `Italic; `Fg_white ]
-    | Details -> [ `Dim; `Fg_white ]
-    | Ok -> [ `Dim; `Fg_green ]
+    | Hint -> [ `Italic ]
+    | Details -> [ `Dim ]
+    | Ok -> [ `Fg_green ]
     | Debug -> [ `Underline; `Fg_bright_cyan ]
     | Success -> [ `Bold; `Fg_green ]
     | Ansi_styles l -> l
@@ -173,9 +173,11 @@ type t =
   ; paragraphs : Style.t Pp.t list
   ; hints : Style.t Pp.t list
   ; annots : Annots.t
+  ; context : string option
+  ; dir : string option
   }
 
-let compare { loc; paragraphs; hints; annots } t =
+let compare { loc; paragraphs; hints; annots; context = _; dir = _ } t =
   let open Ordering.O in
   let= () = Option.compare Loc0.compare loc t.loc in
   let= () = List.compare paragraphs t.paragraphs ~compare:Poly.compare in
@@ -185,17 +187,17 @@ let compare { loc; paragraphs; hints; annots } t =
 
 let equal a b = Ordering.is_eq (compare a b)
 
-let make ?loc ?prefix ?(hints = []) ?(annots = Annots.empty) paragraphs =
+let make ?loc ?prefix ?(hints = []) ?(annots = Annots.empty) ?context ?dir paragraphs =
   let paragraphs =
     match prefix, paragraphs with
     | None, l -> l
     | Some p, [] -> [ p ]
     | Some p, x :: l -> Pp.concat ~sep:Pp.space [ p; x ] :: l
   in
-  { loc; hints; paragraphs; annots }
+  { loc; hints; paragraphs; annots; context; dir }
 ;;
 
-let pp { loc; paragraphs; hints; annots = _ } =
+let pp { loc; paragraphs; hints; annots = _; context; dir = _ } =
   let open Pp.O in
   let paragraphs =
     match hints with
@@ -226,6 +228,12 @@ let pp { loc; paragraphs; hints; annots = _ } =
            (Pp.textf "File %S, %s, characters %d-%d:" start.pos_fname lnum start_c stop_c))
       :: paragraphs
   in
+  let paragraphs =
+    match context with
+    | None | Some "default" | Some ".sandbox" -> paragraphs
+    | Some context ->
+      Pp.box (Pp.tag Style.Loc (Pp.textf "Context: %s" context)) :: paragraphs
+  in
   Pp.vbox (Pp.concat_map paragraphs ~sep:Pp.nop ~f:(fun pp -> Pp.seq pp Pp.cut))
 ;;
 
@@ -270,7 +278,9 @@ let levenshtein_distance s t =
 
 let did_you_mean s ~candidates =
   let candidates =
-    List.filter candidates ~f:(fun candidate -> levenshtein_distance s candidate < 3)
+    List.filter candidates ~f:(fun candidate ->
+      let distance = levenshtein_distance s candidate in
+      0 < distance && distance < 3)
   in
   match candidates with
   | [] -> []
@@ -279,7 +289,7 @@ let did_you_mean s ~candidates =
 
 let to_string t =
   let full_error = Format.asprintf "%a" Pp.to_fmt (pp { t with loc = None }) in
-  match String.drop_prefix ~prefix:"Error: " full_error with
+  match String.drop_prefix ~prefix:"Error:" full_error with
   | None -> full_error
   | Some error -> String.trim error
 ;;
@@ -303,3 +313,9 @@ let command cmd =
     ; Pp.verbatim "'"
     ]
 ;;
+
+let aligned_message ~left:(left_tag, left_string) ~right =
+  let open Pp.O in
+  let left_padded = Printf.sprintf "%12s" left_string in
+  Pp.tag left_tag (Pp.verbatim left_padded) ++ Pp.char ' ' ++ right
+;;
