@@ -15,11 +15,11 @@ type rules = Rules.t
 module Rules = struct
   type t =
     { build_dir_only_sub_dirs : Build_only_sub_dirs.t
-    (** Sub-directories that don't exist in the source tree but exists in
+      (** Sub-directories that don't exist in the source tree but exists in
         the build directory. This is for internal directories such as
         [.dune] or [.ppx]. *)
     ; directory_targets : Loc.t Path.Build.Map.t
-    (** Directories that are target of a rule. For each directory target,
+      (** Directories that are target of a rule. For each directory target,
         give the location of the rule that generates it. The keys in this
         map must correspond exactly to the set of directory targets that
         will be produces by [rules]. The values should be the locations of
@@ -119,9 +119,7 @@ module type Build_config = sig
   (** Initialise the build system. This must be called before running the build
       system and only once. *)
   val set
-    :  action_runner:(Action_exec.input -> Action_runner.t option)
-    -> action_runners:(unit -> Action_runner.t list)
-    -> stats:Dune_stats.t option
+    :  stats:Dune_stats.t option
     -> contexts:(Build_context.t * Context_type.t) list Memo.Lazy.t
     -> promote_source:
          (chmod:(int -> int)
@@ -156,8 +154,6 @@ module type Build_config = sig
     ; implicit_default_alias : Path.Build.t -> unit Action_builder.t option Memo.t
     ; execution_parameters : dir:Path.Build.t -> Execution_parameters.t Memo.t
     ; source_tree : (module Source_tree)
-    ; action_runner : Action_exec.input -> Action_runner.t option
-    ; action_runners : unit -> Action_runner.t list
     ; shared_cache : (module Dune_cache.Shared.S)
     ; write_error_summary : Build_system_error.Set.t -> unit Fiber.t
     }
