@@ -3,13 +3,12 @@
 
    This name could probably be chosen to be a bit more informative. *)
 
-open! Stdune
-open! Import
+open Import
 
 type t = private
   { name : Context_name.t
   ; build_dir : Path.Build.t
-  ; host : Context_name.t option
   }
 
-val create : name:Context_name.t -> host:Context_name.t option -> t
+val create : name:Context_name.t -> t
+val of_build_path : Path.Build.t -> t option
