@@ -1,3 +1,17 @@
+(*
+   This file includes code from the 0install library. It is distributed under
+   the LGPL-2.1-or-later licence. See src/sdune_pkg/COPYING.md for the full
+   license.
+
+   Copyright (C) 2013, Thomas Leonard
+   See the README file for details, or visit http://0install.net.
+
+   Files:
+   - Includes solver_core.ml and solver_core.mli files
+   - Includes diagnostics.ml and diagnostics.mli
+   - Remove usage from the S module from the s.ml file
+*)
+
 open Import
 open Fiber.O
 
@@ -128,7 +142,7 @@ module Context = struct
     ; version_preference
     ; local_packages
     ; pinned_packages
-    ; solver_env
+    ; solver_env = Solver_env.add_sentinel_values_for_unset_platform_vars solver_env
     ; dune_version = Dune_dep.version
     ; stats_updater
     ; candidates_cache
@@ -952,11 +966,8 @@ module Solver = struct
            Fiber.return None
          | Some sels ->
            let nb_avoids sels =
-             Input.Role.Map.fold
-               ~init:0
-               ~f:(fun sel count ->
-                 if Input.Impl.avoid sel.impl then count + 1 else count)
-               sels
+             Input.Role.Map.fold sels ~init:0 ~f:(fun sel count ->
+               if Input.Impl.avoid sel.impl then count + 1 else count)
            in
            let upper = nb_avoids sels in
            (* There exists a solution, using at least 1 and at most [upper]
@@ -1458,10 +1469,32 @@ let solve_package_list packages ~context =
 module Solver_result = struct
   type t =
     { lock_dir : Lock_dir.t
-    ; files : File_entry.t Package_name.Map.Multi.t
+    ; files : File_entry.t Package_version.Map.Multi.t Package_name.Map.t
     ; pinned_packages : Package_name.Set.t
     ; num_expanded_packages : int
     }
+
+  let merge a b =
+    let lock_dir = Lock_dir.merge_conditionals a.lock_dir b.lock_dir in
+    let files =
+      Package_name.Map.union a.files b.files ~f:(fun _ a b ->
+        Some
+          (Package_version.Map.union a b ~f:(fun _ a b ->
+             (* The package is present in both solutions at the same version. Make
+             sure its associated files are the same in both instances. *)
+             if not (List.equal File_entry.equal a b)
+             then
+               Code_error.raise
+                 "Package files differ between merged solver results"
+                 [ "files_1", Dyn.list File_entry.to_dyn a
+                 ; "files_2", Dyn.list File_entry.to_dyn b
+                 ];
+             Some a)))
+    in
+    let pinned_packages = Package_name.Set.union a.pinned_packages b.pinned_packages in
+    let num_expanded_packages = a.num_expanded_packages + b.num_expanded_packages in
+    { lock_dir; files; pinned_packages; num_expanded_packages }
+  ;;
 end
 
 let reject_unreachable_packages =
@@ -1505,7 +1538,13 @@ let reject_unreachable_packages =
           Code_error.raise
             "package is both local and returned by solver"
             [ "name", Package_name.to_dyn name ]
-        | Some (pkg : Lock_dir.Pkg.t), None -> Some (List.map pkg.depends ~f:snd)
+        | Some (pkg : Lock_dir.Pkg.t), None ->
+          Some
+            (Lock_dir.Conditional_choice.choose_for_platform
+               pkg.depends
+               ~platform:solver_env
+             |> Option.value ~default:[]
+             |> List.map ~f:(fun (depend : Lock_dir.Dependency.t) -> depend.name))
         | None, Some (pkg : Local_package.For_solver.t) ->
           let deps =
             match
@@ -1578,13 +1617,17 @@ let reject_unreachable_packages =
 let files resolved_packages =
   Resolved_package.get_opam_package_files resolved_packages
   >>| List.map2 resolved_packages ~f:(fun resolved_package entries ->
-    let package_name =
-      Resolved_package.package resolved_package
-      |> OpamPackage.name
-      |> Package_name.of_opam_package_name
-    in
-    package_name, entries)
-  >>| List.filter ~f:(fun (_, entries) -> List.is_non_empty entries)
+    let package = Resolved_package.package resolved_package in
+    package, entries)
+  >>| List.filter_map ~f:(fun (package, entries) ->
+    if List.is_empty entries
+    then None
+    else (
+      let package_name = OpamPackage.name package |> Package_name.of_opam_package_name in
+      let package_version =
+        OpamPackage.version package |> Package_version.of_opam_package_version
+      in
+      Some (package_name, Package_version.Map.singleton package_version entries)))
   >>| Package_name.Map.of_list_exn
 ;;
 
@@ -1613,6 +1656,7 @@ let solve_lock_dir
       ~pins:pinned_packages
       ~constraints
       ~selected_depopts
+      ~portable_lock_dir
   =
   let pinned_package_names = Package_name.Set.of_keys pinned_packages in
   let stats_updater = Solver_stats.Updater.init () in
@@ -1653,12 +1697,13 @@ let solve_lock_dir
   >>= function
   | Error _ as e -> Fiber.return e
   | Ok solution ->
+    let is_dune name = Package_name.equal Dune_dep.name name in
     (* don't include local packages or dune in the lock dir *)
     let opam_packages_to_lock =
       let is_local_package = Package_name.Map.mem local_packages in
       List.filter solution ~f:(fun package ->
         let name = OpamPackage.name package |> Package_name.of_opam_package_name in
-        (not (is_local_package name)) && not (Package_name.equal Dune_dep.name name))
+        (not (is_local_package name)) && not (is_dune name))
     in
     let* candidates_cache = Fiber_cache.to_table context.candidates_cache in
     let resolve_package name version =
@@ -1683,7 +1728,8 @@ let solve_lock_dir
             version_by_package_name
             opam_package
             ~pinned:(Package_name.Set.mem pinned_package_names name)
-            resolved_package)
+            resolved_package
+            ~portable_lock_dir)
       in
       match Package_name.Map.of_list_map pkgs ~f:(fun pkg -> pkg.info.name, pkg) with
       | Error (name, _pkg1, _pkg2) ->
@@ -1723,39 +1769,41 @@ let solve_lock_dir
           ~hints:[ Pp.text "add a conflict" ]
     in
     let lock_dir =
+      Package_name.Map.iter
+        pkgs_by_name
+        ~f:(fun { Lock_dir.Pkg.depends; info = { name; _ }; _ } ->
+          Lock_dir.Conditional_choice.choose_for_platform depends ~platform:solver_env
+          |> Option.iter ~f:(fun depends ->
+            List.iter depends ~f:(fun { Lock_dir.Dependency.name = dep_name; loc } ->
+              if (not (is_dune dep_name)) && Package_name.Map.mem local_packages dep_name
+              then
+                User_error.raise
+                  ~loc
+                  [ Pp.textf
+                      "Dune does not support packages outside the workspace depending on \
+                       packages in the workspace. The package %S is not in the workspace \
+                       but it depends on the package %S which is in the workspace."
+                      (Package_name.to_string name)
+                      (Package_name.to_string dep_name)
+                  ])));
       let expanded_solver_variable_bindings =
         let stats = Solver_stats.Updater.snapshot stats_updater in
         Solver_stats.Expanded_variable_bindings.of_variable_set
           stats.expanded_variables
           solver_env
       in
-      Package_name.Map.iter
-        pkgs_by_name
-        ~f:(fun { Lock_dir.Pkg.depends; info = { name; _ }; _ } ->
-          List.iter depends ~f:(fun (loc, dep_name) ->
-            if Package_name.Map.mem local_packages dep_name
-            then
-              User_error.raise
-                ~loc
-                [ Pp.textf
-                    "Dune does not support packages outside the workspace depending on \
-                     packages in the workspace. The package %S is not in the workspace \
-                     but it depends on the package %S which is in the workspace."
-                    (Package_name.to_string name)
-                    (Package_name.to_string dep_name)
-                ]));
       Lock_dir.create_latest_version
         pkgs_by_name
         ~local_packages:(Package_name.Map.values local_packages)
         ~ocaml
         ~repos:(Some repos)
         ~expanded_solver_variable_bindings
+        ~solved_for_platform:(Some solver_env)
     in
     let+ files =
-      List.map opam_packages_to_lock ~f:(fun package ->
-        let name = Package_name.of_opam_package_name (OpamPackage.name package) in
-        let version = OpamPackage.version package in
-        resolve_package name version)
+      Package_name.Map.to_list_map pkgs_by_name ~f:(fun name (package : Lock_dir.Pkg.t) ->
+        Package_version.to_opam_package_version package.info.version
+        |> resolve_package name)
       |> files
     in
     Ok
