@@ -22,7 +22,6 @@ end
 
 include struct
   open Decl
-  module Build_outcome_with_diagnostics = Build_outcome_with_diagnostics
   module Status = Status
 end
 
@@ -108,8 +107,8 @@ module Run = struct
   ;;
 end
 
-type 'a pending_build_action =
-  | Build of 'a list * Dune_engine.Scheduler.Run.Build_outcome.t Fiber.Ivar.t
+type 'build_arg pending_build_action =
+  | Build of 'build_arg list * Dune_engine.Scheduler.Run.Build_outcome.t Fiber.Ivar.t
 
 module Client = Stdune.Unit
 
@@ -190,10 +189,10 @@ end = struct
   ;;
 end
 
-type 'a t =
+type 'build_arg t =
   { config : Run.t
-  ; pending_build_jobs : ('a list * Build_outcome.t Fiber.Ivar.t) Job_queue.t
-  ; parse_build : string -> 'a
+  ; pending_build_jobs : ('build_arg list * Build_outcome.t Fiber.Ivar.t) Job_queue.t
+  ; parse_build_arg : string -> 'build_arg
   ; mutable clients : Clients.t
   }
 
@@ -219,7 +218,7 @@ let get_current_diagnostic_errors () =
     | `Diagnostic compound_user_error -> Some compound_user_error)
 ;;
 
-let handler (t : _ t Fdecl.t) handle : 'a Dune_rpc_server.Handler.t =
+let handler (t : _ t Fdecl.t) handle : 'build_arg Dune_rpc_server.Handler.t =
   let on_init session (_ : Initialize.Request.t) =
     let t = Fdecl.get t in
     let client = () in
@@ -333,11 +332,11 @@ let handler (t : _ t Fdecl.t) handle : 'a Dune_rpc_server.Handler.t =
     let build _session targets =
       let server = Fdecl.get t in
       let ivar = Fiber.Ivar.create () in
-      let targets = List.map targets ~f:server.parse_build in
+      let targets = List.map targets ~f:server.parse_build_arg in
       let* () = Job_queue.write server.pending_build_jobs (targets, ivar) in
       let+ build_outcome = Fiber.Ivar.read ivar in
       match (build_outcome : Build_outcome.t) with
-      | Success -> Build_outcome_with_diagnostics.Success
+      | Success -> Dune_rpc_private.Build_outcome_with_diagnostics.Success
       | Failure -> Failure (get_current_diagnostic_errors ())
     in
     Handler.implement_request rpc Decl.build build
@@ -389,6 +388,13 @@ let handler (t : _ t Fdecl.t) handle : 'a Dune_rpc_server.Handler.t =
     in
     Handler.implement_request rpc Procedures.Public.diagnostics f
   in
+  let () =
+    let f _ files =
+      Promote.Diff_promotion.promote_files_registered_in_last_run files;
+      Fiber.return Dune_rpc_private.Build_outcome_with_diagnostics.Success
+    in
+    Handler.implement_request rpc Dune_rpc.Procedures.Public.promote_many f
+  in
   let () =
     let f _ path =
       let files = For_handlers.source_path_of_string path in
@@ -396,7 +402,7 @@ let handler (t : _ t Fdecl.t) handle : 'a Dune_rpc_server.Handler.t =
         (These ([ files ], ignore));
       Fiber.return ()
     in
-    Handler.implement_request rpc Procedures.Public.promote f
+    Handler.implement_request rpc Dune_rpc_private.Procedures.Public.promote f
   in
   let () =
     let f _ () = Fiber.return Path.Build.(to_string root) in
@@ -406,7 +412,7 @@ let handler (t : _ t Fdecl.t) handle : 'a Dune_rpc_server.Handler.t =
   rpc
 ;;
 
-let create ~lock_timeout ~registry ~root ~handle stats ~parse_build =
+let create ~lock_timeout ~registry ~root ~handle stats ~parse_build_arg =
   let t = Fdecl.create Dyn.opaque in
   let pending_build_jobs = Job_queue.create () in
   let handler = Dune_rpc_server.make (handler t handle) in
@@ -444,7 +450,7 @@ let create ~lock_timeout ~registry ~root ~handle stats ~parse_build =
     ; server_ivar = Fiber.Ivar.create ()
     }
   in
-  let res = { config; pending_build_jobs; clients = Clients.empty; parse_build } in
+  let res = { config; pending_build_jobs; clients = Clients.empty; parse_build_arg } in
   Fdecl.set t res;
   res
 ;;
