@@ -198,7 +198,6 @@ module DB = struct
         ~resolve
         ~resolve_lib_id
         ~all:(fun () -> Lib_name.Map.keys by_name |> Memo.return)
-        ~lib_config
         ~instrument_with
   ;;
 
@@ -232,7 +231,7 @@ module DB = struct
       | None -> Memo.return Lib.DB.Resolve_result.not_found
       | Some rt -> resolve_redirect_to t rt
     in
-    fun t ~installed_libs ~lib_config stanzas ->
+    fun t ~installed_libs stanzas ->
       let by_name, by_id =
         List.fold_left
           stanzas
@@ -292,21 +291,20 @@ module DB = struct
         ~resolve
         ~resolve_lib_id
         ~all:(fun () -> Lib_name.Map.keys by_name |> Memo.return)
-        ~lib_config
         ()
   ;;
 
   module Path_source_map_traversals = Memo.Make_parallel_map (Path.Source.Map)
 
   let scopes_by_dir
-    ~build_dir
-    ~lib_config
-    ~projects_by_root
-    ~public_libs
-    ~instrument_with
-    context
-    stanzas
-    coq_stanzas
+        ~build_dir
+        ~lib_config
+        ~projects_by_root
+        ~public_libs
+        ~instrument_with
+        context
+        stanzas
+        coq_stanzas
     =
     let stanzas_by_project_dir =
       List.map stanzas ~f:(fun (dir, stanza) ->
@@ -343,7 +341,6 @@ module DB = struct
   ;;
 
   let create ~context ~projects_by_root stanzas coq_stanzas =
-    let open Memo.O in
     let t = Fdecl.create Dyn.opaque in
     let* context = Context.DB.get context in
     let build_dir = Context.build_dir context in
@@ -354,7 +351,7 @@ module DB = struct
     let instrument_with = Context.instrument_with context in
     let+ public_libs =
       let+ installed_libs = Lib.DB.installed context in
-      public_libs t ~instrument_with ~lib_config ~installed_libs stanzas
+      public_libs t ~instrument_with ~installed_libs stanzas
     in
     let by_dir =
       scopes_by_dir
@@ -447,6 +444,54 @@ module DB = struct
       | Deprecated_library_name { old_name = old_public_name, _; _ } ->
         Public_lib.loc old_public_name
     ;;
+
+    module Set = struct
+      type t =
+        { libraries : Lib.Local.t list
+        ; deprecated_library_names : Deprecated_library_name.t list
+        }
+
+      let empty = { libraries = []; deprecated_library_names = [] }
+
+      let of_list =
+        let by_name x = Lib.Local.info x |> Lib_info.name in
+        fun xs ->
+          let libraries, deprecated_library_names =
+            List.partition_map xs ~f:(function
+              | Library l -> Left l
+              | Deprecated_library_name l -> Right l)
+          in
+          { libraries =
+              List.sort libraries ~compare:(fun x y ->
+                Lib_name.compare (by_name x) (by_name y))
+          ; deprecated_library_names =
+              List.sort
+                deprecated_library_names
+                ~compare:(fun { old_name = old_public_name, _; _ } y ->
+                  Lib_name.compare
+                    (Public_lib.name old_public_name)
+                    (Public_lib.name (fst y.old_name)))
+          }
+      ;;
+
+      let fold { libraries; deprecated_library_names } ~init ~f =
+        let init =
+          List.fold_left ~init libraries ~f:(fun acc lib -> f (Library lib) acc)
+        in
+        List.fold_left deprecated_library_names ~init ~f:(fun acc dep ->
+          f (Deprecated_library_name dep) acc)
+      ;;
+
+      let partition_map t ~f =
+        let l, r =
+          fold t ~init:([], []) ~f:(fun x (l, r) ->
+            match f x with
+            | Left x -> x :: l, r
+            | Right x -> l, x :: r)
+        in
+        List.(rev l, rev r)
+      ;;
+    end
   end
 
   let lib_entries_of_package =
@@ -489,11 +534,7 @@ module DB = struct
             Memo.return ((name, Lib_entry.Deprecated_library_name d) :: acc)
           | _ -> Memo.return acc)
       in
-      Package.Name.Map.of_list_multi libs
-      |> Package.Name.Map.map
-           ~f:
-             (List.sort ~compare:(fun a b ->
-                Lib_name.compare (Lib_entry.name a) (Lib_entry.name b)))
+      Package.Name.Map.of_list_multi libs |> Package.Name.Map.map ~f:Lib_entry.Set.of_list
     in
     let per_context =
       Per_context.create_by_name ~name:"scope-db" (fun ctx ->
@@ -508,11 +549,11 @@ module DB = struct
     in
     fun (ctx : Context_name.t) pkg_name ->
       let+ map = per_context ctx in
-      match Package.Name.Map.Multi.find map pkg_name with
-      | ([] | [ _ ]) as xs -> xs
-      | libs ->
+      match Package.Name.Map.find map pkg_name with
+      | None -> Lib_entry.Set.empty
+      | Some libs ->
         let _by_name =
-          List.fold_left libs ~init:Lib_name.Map.empty ~f:(fun by_name entry2 ->
+          Lib_entry.Set.fold libs ~init:Lib_name.Map.empty ~f:(fun entry2 by_name ->
             let public_name = Lib_entry.name entry2 in
             Lib_name.Map.update by_name public_name ~f:(function
               | None -> Some entry2
