@@ -11,17 +11,17 @@ let dummy_results
       total_number_of_transitive
   =
   List.init (total_number_of_immediate - number_of_immediate) ~f:(fun _ ->
-    Dune_pkg_outdated.For_tests.package_is_best_candidate)
+    Dune_pkg.Outdated.For_tests.package_is_best_candidate)
   @ List.init number_of_immediate ~f:(fun i ->
-    Dune_pkg_outdated.For_tests.better_candidate
+    Dune_pkg.Outdated.For_tests.better_candidate
       ~is_immediate_dep_of_local_package:true
       ~name:(sprintf "foo%d" i)
       ~newer_version:(Dune_pkg.Package_version.of_string "2.0.0")
       ~outdated_version:(Dune_pkg.Package_version.of_string "1.0.0"))
   @ List.init (total_number_of_transitive - number_of_transitive) ~f:(fun _ ->
-    Dune_pkg_outdated.For_tests.package_is_best_candidate)
+    Dune_pkg.Outdated.For_tests.package_is_best_candidate)
   @ List.init number_of_transitive ~f:(fun i ->
-    Dune_pkg_outdated.For_tests.better_candidate
+    Dune_pkg.Outdated.For_tests.better_candidate
       ~is_immediate_dep_of_local_package:false
       ~name:(sprintf "bar%d" i)
       ~newer_version:(Dune_pkg.Package_version.of_string "2.0.0")
@@ -66,7 +66,7 @@ let test_message
   in
   let lock_dir_path = Stdune.Path.Source.of_string "dune.lock" in
   let message =
-    Dune_pkg_outdated.For_tests.explain_results ~transitive ~lock_dir_path results
+    Dune_pkg.Outdated.For_tests.explain_results ~transitive ~lock_dir_path results
   in
   Console.print (List.map ~f:show_styles_of_line message)
 ;;
@@ -286,7 +286,7 @@ let test_entire_output
       total_number_of_transitive
   in
   let lock_dir_path = Stdune.Path.Source.of_string "dune.lock" in
-  let message = Dune_pkg_outdated.For_tests.pp ~transitive ~lock_dir_path results in
+  let message = Dune_pkg.Outdated.For_tests.pp ~transitive ~lock_dir_path results in
   Console.print [ message ]
 ;;
 
